"""
Monkey patch rekey into the list, dict, and set built-in types.

Warning, this depends on the forbiddenfruit library, which
is self proclaimed to be hacky and not production ready.
Use at your own discretion.

usage:  just import this package and you're good to go,
eg. `import rekey.native`
"""

from .rekey import rekey

try:
    import forbiddenfruit
except ImportError:
    raise ImportError(
      'manual install needed: `pip install forbiddenfruit`'
    )

__all__ = [ 'install', 'uninstall' ]


cursed_types = set()

def install(types):
    for _type in types:
        forbiddenfruit.curse(_type, 'rekey', rekey)
        cursed_types.add(_type)


def uninstall(types = None):
    if types is None:
        # uninstall everything
        types = list(cursed_types)

    for _type in types:
        forbiddenfruit.reverse(_type, 'rekey')
        cursed_types.remove(_type)


# wire it up
install([ dict, list, set ])



from .rekey import rekey


class Rekeyable():
    def rekey(self, key_handle=None, value_handle=None):
        return rekey(self, key_handle, value_handle)


class RekeyableList(list, Rekeyable): pass
class RekeyableDict(dict, Rekeyable): pass
class RekeyableSet(set, Rekeyable): pass



import __builtin__

from pluckit.pluckit import pluckit


def rekey(obj, key_handle, value_handle = None):
    supported_types = [list, dict, set]
    if not any([ isinstance(obj, t) for t in supported_types ]):
        raise ValueError('type not supported: %s' % type(obj))

    # determine return type
    if key_handle or hasattr(obj, 'keys'):
        res = {}
    else:
        res = []

    # figure out how to iterate
    if hasattr(obj, 'items'):
        _iter = obj.items()
    else:
        _iter = obj

    # determine how to unpack items into key / value pairs
    if hasattr(obj, 'keys'):
        kv_fn = lambda items: items
    else:
        # no key, only value
        kv_fn = lambda items: (None, items)

    # rekey
    for items in _iter:
        # unpack key / value tuple
        key, value = kv_fn(items)

        # set default result values
        new_key, new_value = key, value

        # grab new key / value
        if key_handle != None:
            new_key = pluckit(value, key_handle)

        if value_handle != None:
            new_value = pluckit(value, value_handle)

        # store results
        if type(res) == list:
            res.append(new_value)
        else:
            res[new_key] = new_value

    return res
